    function dXdt = f(t,X,Param)
    
        %% assigning values to SV
        ii=1;
        tot_cell_number = Param.tot_cell_number;
        
        IP3 = X(ii:tot_cell_number)';
        Ca_c = X(ii*tot_cell_number+1:(ii+1)*tot_cell_number)'; ii=ii+1;
        h = X(ii*tot_cell_number+1:(ii+1)*tot_cell_number)'; ii=ii+1;
        co = X(ii*tot_cell_number+1:(ii+1)*tot_cell_number)'; ii=ii+1;
        o1 = X(ii*tot_cell_number+1:(ii+1)*tot_cell_number)'; ii=ii+1;
        o2 = X(ii*tot_cell_number+1:(ii+1)*tot_cell_number)'; ii=ii+1;
        c2 = X(ii*tot_cell_number+1:(ii+1)*tot_cell_number)'; ii=ii+1;
        s = X(ii*tot_cell_number+1:(ii+1)*tot_cell_number)'; ii=ii+1;
        
        dXdt = zeros(numel(X),1);
        
        %% model parameters and options
        IP3_stim = Param.IP3_stim;
        laser_stim = Param.laser_stim;
        laser_on = Param.laser_on;
        laser_off = Param.laser_off;
        T = Param.T;
        delta = Param.delta;
        IP3_stim_onset = Param.IP3_stim_onset;
        IP3_stim_end = Param.IP3_stim_end;
        IP3_level_basal = Param.IP3_level_basal;
        IP3_level_stim = Param.IP3_level_stim;
        p1 = Param.p1;
        gd1 = Param.gd1;
        e12 = Param.e12;
        e21 = Param.e21;
        p2 = Param.p2;
        gd2 = Param.gd2;
        gr = Param.gr;
        tau = Param.tau;
        gamma = Param.gamma;
        Vm = Param.Vm;
        EChR2 = Param.EChR2;
        conversion_Fact = Param.conversion_Fact;
        G_V = Param.G_V;
        g1 = Param.g1;
        vdelta = Param.vdelta;
        KdCa = Param.KdCa;
        KIP3 = Param.KIP3;
        xCCE = Param.xCCE;
        hCCE = Param.hCCE;
        c1 = Param.c1;
        V_SERCA = Param.V_SERCA;
        Kp = Param.Kp;
        d1 = Param.d1;
        d5 = Param.d5;
        v1 = Param.v1;
        v2 = Param.v2;
        kout = Param.kout;
        lambda = Param.lambda;
        epsilon = Param.epsilon;
        Jin_Astro = Param.Jin_Astro;
        a = Param.a;
        d2 = Param.d2;
        d3 = Param.d3;
        bt = Param.bt;
        K = Param.K;
        neighbors = Param.neighbors;
        IP3_stim_cell_indx = Param.IP3_stim_cell_indx;
        laser_stim_cell_indx = Param.laser_stim_cell_indx;
        DIP3 = Param.DIP3;
        DCa = Param.DCa;
        
        %% stimulation protocol
        
        % ---------- IP3 stimulus
        X_IP3 = IP3_level_basal.*ones(1,tot_cell_number);
        
        if IP3_stim
            if t>=IP3_stim_onset && t<=IP3_stim_end
                X_IP3(IP3_stim_cell_indx) = IP3_level_stim;  %X_IP3 is assigned the IP3 stimulation paradigm defined by the user
            end
        end
        
        % ----------- continuous stimulation
        theta_s = zeros(1,tot_cell_number);
        
        % ------------ laser (squarewave) stimulus
        if laser_stim
            if t>=laser_on && t<=laser_off              % time window of stimulation
                tmp = mod(t,T);
                theta_s(laser_stim_cell_indx) = (tmp < delta);
            else
                theta_s(laser_stim_cell_indx) = 0;
            end
        end
        
        s0theta = 0.5*(1+tanh(120*(theta_s-0.1)));      % from Stefanescu et al
        
        %% gap junctional fluxes
        J_gjCa = zeros(1,tot_cell_number);
        J_gjIP3 = zeros(1,tot_cell_number);
        for i = 1:tot_cell_number
            for j = 1:nnz(neighbors(i,:))
                J_gjCa(i) = J_gjCa(i) + DCa*(Ca_c(i) - Ca_c(neighbors(i,j)));
                J_gjIP3(i) = J_gjIP3(i) + DIP3*(IP3(i) - IP3(neighbors(i,j)));
            end
        end
        
        %% ChR2 differential equations
        do1dt = p1.*s.*(1 - c2-o1-o2) - (gd1+e12) .* o1 + e21.*o2;  %c1 = 1 - c2-o1-o2
        do2dt = p2.*s.*c2 + e12.*o1 - (gd2+e21) .* o2;
        dc2dt = gd2.*o2 - (p2.*s+gr).*c2;
        dsdt = ((s0theta) - s)/tau;
        
        %% SAM differential equations
        
        %--------- IP3 dynamics
        dIP3dt =  vdelta .*Ca_c.^2./(Ca_c.^2+KdCa.^2) - KIP3.*IP3 + X_IP3 - J_gjIP3; % rate of production of IP3, agonist based/ PLC
        
        %--------- Ca_c dynamics
        v_CCE = xCCE.*hCCE.^2./(hCCE.^2 + ((co - Ca_c)./c1).^2); %influx of Ca2+ ions from the EC space -> happens when ER are depleted
        v_SERCA = V_SERCA .* Ca_c.^2./(Kp.^2 + Ca_c.^2);          % final SERCA component, SERCA pumping into the ER from the cytosol, active transport component
        m_inf = (IP3 ./ (IP3 + d1))  .* (Ca_c ./ (Ca_c + d5));  % gating of the IP3 channel
        v_Rel = ( v1 .*  m_inf.^3.* h.^3 + v2).* (co - (1 + c1) .* Ca_c); % Flow of Ca2+ from the ER to the cytosol - part of IP3 channel dynamics
        v_out = kout .* Ca_c;   % PMCA pump
        I_ChR = g1.*G_V.*(o1+ gamma.*o2).*(Vm - EChR2);
        JinChR = I_ChR.*conversion_Fact;   %ChR2 based Ca2+ influx into the cytosol
        f_c = lambda.*(v_Rel - v_SERCA) + epsilon .* (Jin_Astro + v_CCE - v_out  - JinChR) - J_gjCa; % Ca2+ influx and efflux to and from the cytosol
        theta_fba = (bt.*K)./((K+f_c).^2);
        dCa_cdt = f_c./(1 + theta_fba);
        
        %--------- co dynamics
        dcodt = (epsilon .*(Jin_Astro + v_CCE - v_out - JinChR) - J_gjCa)./(1 + theta_fba); % rate of change of total free Ca2+
        %--------- Gating (h) variable dynamics
        alpha_h = a.*d2.*(IP3+d1)./(IP3+d3);
        beta_h = a.*Ca_c;
        dhdt = alpha_h.*(1-h) - beta_h.*h; %in the paper it is mistakenly written as +
        
        %% assigning output
        ii=1;
        dXdt(ii:tot_cell_number)= dIP3dt;
        dXdt(ii*tot_cell_number+1:(ii+1)*tot_cell_number)= dCa_cdt; ii = ii+1;
        dXdt(ii*tot_cell_number+1:(ii+1)*tot_cell_number)= dhdt; ii = ii+1;
        dXdt(ii*tot_cell_number+1:(ii+1)*tot_cell_number)= dcodt; ii=ii+1;
        dXdt(ii*tot_cell_number+1:(ii+1)*tot_cell_number) = do1dt; ii=ii+1;
        dXdt(ii*tot_cell_number+1:(ii+1)*tot_cell_number) = do2dt; ii=ii+1;
        dXdt(ii*tot_cell_number+1:(ii+1)*tot_cell_number) = dc2dt; ii=ii+1;
        dXdt(ii*tot_cell_number+1:(ii+1)*tot_cell_number) = dsdt;
        
    end