function LogLK = LogLKFunction(ThetaVar,EstimOptions)

% The Likelihood function and the parameters to be estimated are normalized according the good optimization
% practice sugested at Chapter "Practicalities" in:
% Gill P.E., Murray W. and Wright M.H., Practical Optimization. Academic Press, 1981.

global gct    % variable to be used in confun.m

t          = EstimOptions{1};
Z          = EstimOptions{2};
Xf00       = EstimOptions{3};
Pf0        = EstimOptions{4};
Theta0     = EstimOptions{5};
FileNames  = EstimOptions{6};
LKWindow   = EstimOptions{7};
LogLK0     = EstimOptions{8};
nu_stast   = EstimOptions{9};
Xf0Index   = EstimOptions{14};
ThetaIndex = EstimOptions{15};
mpts       = EstimOptions{16};
AbsTol     = EstimOptions{18};
RelTol     = EstimOptions{19};


switch EstimOptions{13},
   case 0,
      Xf  = Xf00;
      Th  = Theta0;
      Par = [Xf00(Xf0Index)' Theta0(ThetaIndex)];  
      NoZeroPar = find(abs(Par)>eps);
        ZeroPar = find(abs(Par)<eps);
        
      Par(NoZeroPar)= ThetaVar(NoZeroPar) .* Par(NoZeroPar);
      Par(ZeroPar)  = ThetaVar(ZeroPar) - 1;
      Xf(Xf0Index)  = Par(1:length(Xf0Index));
      Th(ThetaIndex)= Par(1+length(Xf0Index):end);
      [Xf,Pf,Xp,Pp,Inn,LK,PInn,mpts]=LLfilter(t,Z,Xf,Pf0,Th,FileNames,LKWindow,mpts,AbsTol,RelTol);
   case 1,
      Th  = Theta0;
      Par = Theta0(ThetaIndex);         % cada elemento de Par tiene su correspondiente elemento en ThetaVar
      NoZeroPar = find(abs(Par)>eps);
        ZeroPar = find(abs(Par)<eps);

      Par(NoZeroPar)= ThetaVar(NoZeroPar) .* Par(NoZeroPar);
      Par(ZeroPar)  = ThetaVar(ZeroPar) - 1;
      Th(ThetaIndex)= Par;
      [Xf,Pf,Xp,Pp,Inn,LK,PInn,mpts]=LLfilter(t,Z,Xf00,Pf0,Th,FileNames,LKWindow,mpts,AbsTol,RelTol);
   case 2,
      Xf  = Xf00;
      Par = Xf00(Xf0Index);            % cada elemento de Par tiene su correspondiente elemento en ThetaVar
      NoZeroPar = find(abs(Par)>eps);
        ZeroPar = find(abs(Par)<eps);
      
      Par(NoZeroPar)= ThetaVar(NoZeroPar)' .* Par(NoZeroPar);
      Par(ZeroPar)  = ThetaVar(ZeroPar) - 1;
      Xf(Xf0Index)= Par;
      [Xf,Pf,Xp,Pp,Inn,LK,PInn,mpts]=LLfilter(t,Z,Xf,Pf0,Theta0,FileNames,LKWindow,mpts,AbsTol,RelTol);
   end
   
n=size(Inn,1);
for i=1:n
   subplot(n+1,1,i)
   plot(t,Inn(i,:)./sqrt(PInn(:,i,i)'))
end   
subplot(n+1,1,1), title('Standardized fitting-innovation inside the optimization algorithm')

subplot(n+1,1,n+1)
plot(t,mpts(1,:))
Min=num2str(min(mpts(1,2:end)));
Max=num2str(max(mpts(1,2:end)));
Sum=num2str(sum(mpts(1,:)));
subplot(n+1,1,n+1),  title(['mpts,     Min= ',Min,'   Max= ',Max,'  Total= ',Sum])
drawnow

if ~isempty(nu_stast),
  index=find((t>=LKWindow(1)) & (t<=LKWindow(2)));
  firstpoint=index(1);                 % starting point for the computation of the innovations statistics
  lastpoint=index(end);                % last point for the computation of the innovations statistics
  nut=Inn(:,firstpoint:lastpoint)';
  gct=[abs(mean(nut)) diag(cov(nut))'];
  gct=gct-nu_stast;
end

LogLK = LK./abs(LogLK0);   


