function [ms, Ps, ms1, Ps1] = LLpredictionAuto(s,A,f0,B,b,m0,P0)

% This function computes the conditional mean and variance of the equation:
%   dx = f(x)*dt + sum((B*x+b)*dw), x(0)=x0
% evaluated at the time s. 
% Here,f(x)=A*x+a, A and B are constant matrices, and a and b are constant vectors
% These moments satisface the equations:
%   dm/dt = A*m+a,  m(0)=m0
%   dP/dt = A*P + P*A' + sum(B*P*B' + B*m'*b' + b*m*B + b*b' + B*m*m'*B), P(0)=P0
% In the definition of the function LLvariance the variable f0=f(0), i.e., 
% f0 = A*m0+a.

%References
% 1) "Local Linearization filters for non-linear continuous-discrete state 
%     space models with multiplicative noise", Jimenez J.C. & Ozaki
%     Int. J. Control (2003), 76, 1159-1170. 
% 2) "Linear estimation of continuous-discrete linear state space models 
%     with multiplicative noise",  Jimenez J.C. & Ozaki T.
%     Systems & Control Letters (2002), 47, 91-101. See also, Erratum (2003), 49, 161 

d=size(m0,1);
d2=d*d;

I_d=eye(d);
bbT=zeros(d,d);
BK{1}=zeros(d2,d2);
BK{2}=zeros(d2,d);
BK{3}=zeros(d2,d);
m=max([size(B,2) size(b,2)]);
for i=1:m 
   bbT = bbT + b{i}*b{i}';
   BK{1}= BK{1} + kron(B{i},B{i});
   BK{2}= BK{2} + kron(b{i},B{i});
   BK{3}= BK{3} + kron(B{i},b{i}); 
end
a0=f0-A*m0;
v0=zeros(d,1);
LT=[I_d v0];
R=[v0; 1];

AE = kron(I_d,A) + kron(A,I_d) + BK{1};

beta1 = vec(bbT);
beta4 = kron(I_d,a0) + kron(a0,I_d) + BK{2} + BK{3};

beta1 = beta1 + beta4*m0;
beta4 = beta4*LT;

%sugestion: use krylov algorithm instead of expm if d>10

C = [A f0; zeros(1,d+1)];
M = [AE beta1 beta4; zeros(1,d2+d+2); zeros(d+1,d2+1) C];
M = expm(M.*s);
Ps= M(1:d2,1:d2)*vec(P0) + M(1:d2,d2+1) + M(1:d2,d2+2:end)*R;

F3 = M(d2+2:end,d2+2:end);
ms = m0 + F3(1:d,d+1);

if nargout>2,
  MM  = M*M;  

  F3  = MM(d2+2:end,d2+2:end);
  ms1 = m0 + F3(1:d,d+1);

  Ps1 = MM(1:d2,1:d2)*vec(P0) + MM(1:d2,d2+1) + MM(1:d2,d2+2:end)*R;
  Ps1 = reshape(Ps1,d,d);
end  

Ps =reshape(Ps,d,d);



%----------------------------------

function v = vec(x);

[n m]=size(x);
v=reshape(x,n*m,1);
