function [ms, Ps, ms1, Ps1] = LLpredictionAutoAdd(s,A,f0,b,m0,P0)

% This function computes the conditional mean and variance of the equation:
%   dx = f(x)*dt + sum(b)*dw), x(0)=x0
% evaluated at the time s. 
% Here,f(x)=A*x+a, A is a constant matrix, and a and b are vectors
% These moments satisface the equations:
%   dm/dt = A*m+a,  m(0)=m0
%   dP/dt = A*P + P*A' + sum(b*b'), P(0)=P0
% In the definition of the function LLvariance the variable f0=f(0), i.e., 
% f0 = A*m0+a.

%Reference
% "Local Linearization filters for non-linear continuous-discrete state 
%  space models with multiplicative noise", Jimenez J.C. & Ozaki
%  Int. J. Control (2003), 76, 1159-1170. 

d=size(m0,1);

a0=f0-A*m0;
m=size(b,2);
bbT=zeros(d);
for i=1:m 
   bbT = bbT + b{i}*b{i}';
end
C=[A a0 a0*m0'+0.5.*bbT f0; zeros(1,d+1) f0' 0; zeros(d,d+1) -A' zeros(d,1); zeros(1,2*d+2)];
M = expm(C.*s);
F1=M(1:d,1:d);
g1=M(1:d,2*d+2);
H1=M(1:d,d+2:end-1);

Ps = H1*F1';
Ps = F1*P0*F1' + Ps + Ps';
ms = m0 + g1;

if nargout>2,
  MM=M*M;
  F1=MM(1:d,1:d);
  g1=MM(1:d,2*d+2);
  H1=MM(1:d,d+2:end-1);

  Ps1 = H1*F1';
  Ps1 = F1*P0*F1' + Ps1 + Ps1';
  ms1 = m0 + g1;
end  
